// RW Pageone Project
// ==================
// Imageboard 'Page 1' reader demo project;  SAVE.EVERYTHING.ANON. :^)

// Filename: ./main_pageone.cpp
// -The program's entry point

#include "incls_all.hpp"

//------------------------------------------------------------------------------
/** Bumpmaster system demo; print various board's Page 1 thread's 5 newest posts
 * @note: this is similar to an IB's index page (as a text-only printout)
 */
int main() try {
  puts("\n SAVE.EVERYTHING.ANON. :^)");

  auto const thrd_lim{10};  // just read each board's Page 1 threads
  auto const post_lim{5};   // just read each threads's 5 newest posts

  // Container of various IB URI strings
  // -try other boards you like, Anon!  :^)
  // clang-format off
  vector<string> const board_uris{
      // NOTE: currently supporting only Lynxchan sites (as of v220521)

      "https://alogs.space/robowaifu/",
      "https://late.city/late/",
      "https://anon.cafe/server/",
      "https://alogs.space/tech/",
      "https://anon.cafe/christian/",

  };
  // clang-format on

  Bumpmaster bump{"bumpmaster"};

  auto const sites_boards = bump.preflight_boards(board_uris);  // prep brd dirs
  auto const pulled_cats = bump.chk_pull_cats_sh(sites_boards);  // check & pull

  for (auto const& [site_nm, boards] : sites_boards) {  //

    /// Sites...

    puts("\n\n     --=^=--\n\n");

    cout << " =======================================\n"
         << "    cycling " << site_nm << " boards...\n"
         << " =======================================\n";

    for (auto const& [thrd_uri, board_path, idx] : boards) {  //

      /// Boards...

      auto const& [site_nm, board_nm] = parse_ib_uri(thrd_uri);

      puts("\n\n     --=^=--\n\n");

      cout << " ---------------------------------------\n"
           << "   reading /" << board_nm << "/'s dolldrops...\n"
           << " ---------------------------------------\n";

      auto const cat_path{board_path / "catalog.json"};
      auto const cat_json = rd_json_file(cat_path);

      // optimize thread reads via 'pulled cats' filter:
      // is board's matching path contained within the 'pulled_cats' container?
      // -if so, set a flag to check up on it's threads, too
      //
      auto cat_it = find_if(
          cbegin(pulled_cats), cend(pulled_cats), [&cat_path](auto const& e) {
            auto const& [pull_cat_uri, pull_cat_path, pull_idx]{e};
            return cat_path == pull_cat_path;
          });
      auto chk_thrds = (cat_it != cend(pulled_cats));  // was path was found?

      /// Threads...

      auto const brd_jsons_path{board_path / "all_jsons"};

      // load new container with catalog's Page 1 threads + posts; in bump-order
      auto const top_thrds = bump.top_bump_thrds_sh(
          cat_json, thrd_uri, brd_jsons_path, thrd_lim, chk_thrds);

      // cycle the top threads
      size_t t_idx{0};                   //
      for (auto const& t : top_thrds) {  //
        puts("\n\n     --=^=--\n");

        cout << "     Bump  " << ++t_idx  // prt thread's data
             << "\n =======================================\n"  //
             << setw(8) << setfill(' ') << t.id << "  :  " << t.subj << "\n\n' "
             << bump.fmt_msg(t.msg) << " '\n";

        /// Posts...

        // Cycle the 'post_lim' count of newest posts
        //
        // calc starting post's thread index
        auto const sz =
            static_cast<int>(t.posts.size());  // cast avoids possible overflow,
        auto const start{sz - post_lim};       //  for this subtraction
        //
        for (auto p_idx{
                 (start >= 0 ? start : 0)};  // ensure we have a valid index,
             p_idx < sz; ++p_idx) {          //  (as sz might be < post_lim)

          auto const& p = t.posts[p_idx];   // index to post
          cout << '\n'                      // prt post's data
               << setw(33) << setfill(' ')  //
               << "r " << (p_idx + 1)
               << "\n  ------------------------------------\n"
               << setw(8) << setfill(' ') << p.id << "  :  " << p.date << "\n' "
               << bump.fmt_msg(p.msg) << " '\n";
        }
      }
    }
  }

  puts("\n\n     --=^=--\n\n");

  puts(" ...you've saved everything right Anon?  :^)\n");

} catch (runtime_error const& e) {
  cerr << "\n  runtime_error '" << e.what() << "' caught in main()\n\n";

} catch (exception const& e) {
  cerr << "\n  exception '" << e.what() << "' caught in main()\n\n";

} catch (...) {
  cerr << "\n  unknown error caught in main()\n\n";
}

//    --=^=--

// "Everyone's talking about IT & the Internet, and how advanced it's getting."

// rw_pageone_demo.v220521
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
